/*
 * hey_manager.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file hey_manager.cc
 *
 * Tool to get/set the active resource broker policy.
 */

#include "config.h"

#include <stdlib.h>

#include <iostream>

#include <HeyParser.hh>

#include <BrokerC.h>

int main(int argc, char *argv[])
{
    int retval = EXIT_FAILURE;

    try
    {
	CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
	HeyParser hp(argc, (const char **)argv);
	const char *prop_name, *prop_value;
	Broker::Manager_var manager;
	const char *manager_ior;
	CORBA::Object_var obj;

	manager_ior = hp.who();
	obj = orb->string_to_object(manager_ior);
	manager = Broker::Manager::_narrow(obj.in());
	if( CORBA::is_nil(manager.in()) )
	{
	    throw CORBA::BAD_PARAM();
	}

	switch( hp.what() )
	{
	case HeyParser::LIST_PROPERTIES:
	    cout << "policy" << endl;
	    retval = EXIT_SUCCESS;
	    break;
	case HeyParser::GET_PROPERTY:
	case HeyParser::SET_PROPERTY:
	    hp.popProperty(prop_name, prop_value);
	    if( strcmp(prop_name, "policy") == 0 )
	    {
		switch( hp.what() )
		{
		case HeyParser::GET_PROPERTY:
		    {
			Broker::Policy_var policy;

			policy = manager->CurrentPolicy();
			if( CORBA::is_nil(policy.in()) )
			{
			    cout << "(none)" << endl;
			}
			else
			{
			    cout << manager->CurrentPolicy()->Name() << endl;
			}
			retval = EXIT_SUCCESS;
		    }
		    break;
		case HeyParser::SET_PROPERTY:
		    {
			Broker::Policy_var policy;
			const char *policy_ior;
			
			policy_ior = hp.to();
			obj = orb->string_to_object(policy_ior);
			policy = Broker::Policy::_narrow(obj.in());
			if( CORBA::is_nil(policy.in()) )
			{
			    throw CORBA::BAD_PARAM();
			}
			manager->CurrentPolicy(policy.in());
			cout << "ok" << endl;
			retval = EXIT_SUCCESS;
		    }
		    break;
		}
	    }
	    else
	    {
		cerr << "Unknown property: " << prop_name << endl;
	    }
	    break;
	default:
	    cerr << "Unknown verb..." << endl;
	    break;
	}
	
	orb->shutdown();
    }
    catch(const HeyParserException &he)
    {
	cerr << "Parse error: " << he << endl;
    }
    catch(const CORBA::SystemException &e)
    {
	cerr << "Caught Exception: " << e << endl;
    }
    catch(...)
    {
	cerr << "Caught an unhandled exception" << endl;
    }

    return( retval );
}
