/*
 * hey_good.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file hey_good.cc
 *
 * Test valid inputs to a HeyParser instance.
 *
 * @sa hey_bad.cc
 */

#include "config.h"

#include <iostream>

#include <assert_pp.h>

#include "HeyParser.hh"

int main(int argc, const char *argv[])
{
    int retval = EXIT_SUCCESS;

    /* Check the verb mappings for list, */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "list",
	};

	HeyParser hp(3, hp_argv);

	require(strcmp(hp.who(), hp_argv[1]) == 0);
	require(hp.what() == HeyParser::LIST_PROPERTIES);
    }

    /* ... get, and */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "get", "prop",
	};

	HeyParser hp(4, hp_argv);
	const char *name, *value = "not-null";

	require(hp.what() == HeyParser::GET_PROPERTY);

	hp.popProperty(name, value);

	require(strcmp(name, hp_argv[3]) == 0);
	require(value == NULL);
    }

    /* ... set. */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "set", "prop", "foo",
	};

	HeyParser hp(5, hp_argv);
	const char *name, *value;

	require(hp.what() == HeyParser::SET_PROPERTY);

	hp.popProperty(name, value);

	require(strcmp(name, hp_argv[3]) == 0);
	require(strcmp(value, hp_argv[4]) == 0);
    }

    /*
     * Make sure 'to' is detected properly for a 'set' with a name/value
     * property and
     */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "set", "prop", "foo", "to", "5",
	};

	HeyParser hp(7, hp_argv);
	const char *name, *value;

	require(hp.what() == HeyParser::SET_PROPERTY);

	hp.popProperty(name, value);

	require(strcmp(name, hp_argv[3]) == 0);
	require(strcmp(value, hp_argv[4]) == 0);
	require(strcmp(hp.to(), hp_argv[6]) == 0);
    }

    /* ... just a name property. */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "set", "prop", "to", "5",
	};

	HeyParser hp(6, hp_argv);
	const char *name, *value;

	require(hp.what() == HeyParser::SET_PROPERTY);

	hp.popProperty(name, value);

	require(strcmp(name, hp_argv[3]) == 0);
	require(strcmp(hp.to(), hp_argv[5]) == 0);
    }

    /* Make sure multiple properties are stack properly. */
    {
	const char *hp_argv[] = {
	    "hey", "ior", "get", "prop", "foo", "of", "prop", "bar",
	};

	HeyParser hp(8, hp_argv);
	const char *name, *value;

	require(hp.what() == HeyParser::GET_PROPERTY);

	hp.popProperty(name, value);
	
	require(strcmp(name, hp_argv[6]) == 0);
	require(strcmp(value, hp_argv[7]) == 0);

	hp.popProperty(name, value);

	require(strcmp(name, hp_argv[3]) == 0);
	require(strcmp(value, hp_argv[4]) == 0);
    }
    
    return( retval );
}
