/*
 * ResourceSetTest.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

import rk.ResourceSet;

/**
 * Test for the rk.ResourceSet class.
 */
public class ResourceSetTest
{
    /**
     * Volatile counter variable for an empty 'for' loop, otherwise, it might
     * get optimized away.
     */
    private static volatile int counter;

    /**
     * Verbosity flag, turned on when any arguments are passed to main.
     */
    public static boolean verbose;
    
    public static void main(String args[])
    {
	if( args.length > 0 )
	{
	    verbose = true;
	}
	
	try
	{
	    long usage, usage2;

	    /* Make sure its working in the first place. */
	    usage = ResourceSet.getUsage();
	    if( verbose )
	    {
		System.out.println("usage = " + usage);
	    }
	    if( usage == 0 )
	    {
		System.err.println("Zero usage?");
		System.exit(1);
	    }

	    /* Chew up some CPU and test again. */
	    for( counter = 0; counter < 100000; counter += 1 );

	    usage2 = ResourceSet.getUsage();
	    if( verbose )
	    {
		System.out.println("usage2 = " + usage2);
	    }
	    if( usage2 <= usage )
	    {
		System.err.println("Bogus usage: old="
				   + usage
				   + "; new="
				   + usage2);
		System.exit(1);
	    }

	    System.exit(0);
	}
	catch(UnsatisfiedLinkError e)
	{
	    /* No librk on this machine. */
	    if( verbose )
	    {
		e.printStackTrace();
	    }
	}
    }
}
