#! /bin/awk

#
# rktimes_sum.awk
#
# Copyright (c) 2003 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is licensed under the terms of the GNU Public License.  
# See the file "license.terms" for restrictions on redistribution 
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

#
# rktimes_sum.awk - Sum the Y values of a group of rktimes output files.
#
# Usage: awk -f rktimes_sum.awk <rktimes-output1> [<rktimes-output2> ...]
#
# The rktimes_sum.awk script is used to merge a group of output files generated
# by a single run of rktimes(1).  Multiple output files can be generated when
# the utility does a fork(2) or spawns a thread.
#
# Example:
#
#   $ awk -f rktimes_offset.awk rktimes-1234.out rktimes-1235.out
#
# See Also:
#
#   rktimes(1), rktimes_sum.awk
#

BEGIN {
    if( ARGC < 2 )
    {
	printf("Usage: awk -f rktimes_sum.awk <rktimes-output1> [<rktimes-output2> ...]\n") > "/dev/stderr";
	exit 1;
    }
    times_count = 0;
    start_count = 0;
}

# Copy the start time to the output and make sure they all agree.
/^\# start: / {
    if( start_count == 0 )
    {
	printf("%s\n", $0);
	start_count += 1;
	start_time = $0;
    }
    else if( start_time != $0 )
    {
	printf("%s: Start time mismatch: %s != %s\n",
	       FILENAME,
	       start_time,
	       $0) > "/dev/stderr";
	times_count = 0;
	exit 1;
    }
}

# Collect and sum the data points.
/[0-9\.\-]+ [0-9\.\-]+/ {
    if( !($1 in usage) )
    {
	times[times_count] = $1;
	times_count += 1;
    }
    usage[$1] = usage[$1] + $2;
}

# Finally, print out everything collected.
END {
    for( lpc = 0; lpc < times_count; lpc++ )
    {
	printf("%f %f\n", times[lpc], usage[times[lpc]]);
    }
}
