#! /bin/sh

RKTIMES=../src/rktimes

$RKTIMES -h > usage.out 2>&1

# Check for no arguments.
$RKTIMES > rktimes_test1_1.sh.tmp 2>&1

cat > rktimes_test1_1.sh.out <<EOF
../src/rktimes: Missing utility to monitor
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_1.sh.out rktimes_test1_1.sh.tmp; then
    echo "rktimes works with no arguments?"
    exit 1
fi

# Check for -o without an argument.
$RKTIMES -o > rktimes_test1_2.sh.tmp 2>&1

cat > rktimes_test1_2.sh.out <<EOF
../src/rktimes: option requires an argument -- o
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_2.sh.out rktimes_test1_2.sh.tmp; then
    echo "-o with no argument works?"
    exit 1
fi

# Check for -n without an argument.
$RKTIMES -n > rktimes_test1_3.sh.tmp 2>&1

cat > rktimes_test1_3.sh.out <<EOF
../src/rktimes: option requires an argument -- n
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_3.sh.out rktimes_test1_3.sh.tmp; then
    echo "-n with no argument works?"
    exit 1
fi

# Check for -n with a long argument.
$RKTIMES -n 12345678901234567 > rktimes_test1_4.sh.tmp 2>&1

cat > rktimes_test1_4.sh.out <<EOF
../src/rktimes: -n value too long (only 16 characters allowed)
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_4.sh.out rktimes_test1_4.sh.tmp; then
    echo "-n with long argument works?"
    exit 1
fi

# Check for -p without an argument.
$RKTIMES -p > rktimes_test1_5.sh.tmp 2>&1

cat > rktimes_test1_5.sh.out <<EOF
../src/rktimes: option requires an argument -- p
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_5.sh.out rktimes_test1_5.sh.tmp; then
    echo "-p with no argument works?"
    exit 1
fi

# Check for -p with a non numeral argument.
$RKTIMES -p foo > rktimes_test1_6.sh.tmp 2>&1

cat > rktimes_test1_6.sh.out <<EOF
../src/rktimes: Invalid -p value: foo
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_6.sh.out rktimes_test1_6.sh.tmp; then
    echo "-p works with string?"
    exit 1
fi

# Check for -p 0.
$RKTIMES -p 0 > rktimes_test1_7.sh.tmp 2>&1

cat > rktimes_test1_7.sh.out <<EOF
../src/rktimes: Invalid -p value: 0
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_7.sh.out rktimes_test1_7.sh.tmp; then
    echo "-p works with 0?"
    exit 1
fi

# Check for -p 65536.
$RKTIMES -p 65536 > rktimes_test1_8.sh.tmp 2>&1

cat > rktimes_test1_8.sh.out <<EOF
../src/rktimes: Invalid -p value: 65536
`cat usage.out`
EOF

if ! cmp -s rktimes_test1_8.sh.out rktimes_test1_8.sh.tmp; then
    echo "-p works with 65536?"
    exit 1
fi
