/*
 * RTServer_busy.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file RTServer_busy.cc
 */

#include "config.h"

#include <assert_pp.h>

#include <iostream>

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "RTServer_busy.hh"

static double MHZ;

RTServer_busy::RTServer_busy(CORBA::ULong deadline_us,
			     CORBA::ULong *compute_us,
			     unsigned int compute_us_length)
{
    require(compute_us != NULL);
    
    this->rts_Deadline = deadline_us;
    this->rts_Compute.data = compute_us;
    this->rts_Compute.length = compute_us_length;
    this->rts_Compute.index = 0;
    MHZ = 850.0; // XXX FindMHZ();
}

typedef signed long long int cycle_time;
typedef double us_time;

// ugh -- can't seem to get accurate MHz on win2k
#ifdef WIN32
#define MHZ_SLOP 5
#else
#define MHZ_SLOP 2
#endif

static us_time get_timeval_us(void)
{
  struct timeval tv;
  us_time t;
  
  gettimeofday (&tv, NULL);
  t = tv.tv_usec + (1000000 * tv.tv_sec);
  return t;
}

static inline cycle_time rdtsc (void)
{
  cycle_time d;
  __asm__ __volatile__ ("rdtsc" : "=A" (d) );
  return d;
}

static double my_fabs (double d)
{
  if (d < 0) {
    return -d;
  } else {
    return d;
  }
}

#if 0
double RTServer_busy::FindMHZ(void)
{
    double last_freq, freq = 0;
    int i;
    int good = 0;
    us_time us_start, us_stop, us_diff;
    cycle_time cycle_start, cycle_stop, cycle_diff;

    setpriority(PRIO_PROCESS, 0, 20);
    
    for( i = 0; i < 100; i++ )
    {
	// estimate MHz
	
	us_start = get_timeval_us ();
	cycle_start = rdtsc ();
	
	/*
	 * ugh -- sleeping permits some laptops to power down their
	 * processors, which screws up the measurements -- so busy-wait
	 * instead
	 */
#if 0
	usleep (1000);
#else
	{
	    volatile int x;
	    
	    for (x=0; x < 5*1000*1000; x++);
	}
#endif
	
	us_stop = get_timeval_us ();
	cycle_stop = rdtsc ();
	
	us_diff = us_stop - us_start;
	cycle_diff = cycle_stop - cycle_start;
	
	last_freq = freq;
	freq = cycle_diff / us_diff;
	
	// wait for three successive values within range
	
	if (my_fabs (last_freq - freq) < MHZ_SLOP) {
	    good++;
	    if (good == 3) {
		setpriority(PRIO_PROCESS, 0, 0);
		return freq;
	    }
	} else {
	    good = 0;
	}
    }
    
    printf ("couldn't determine MHz of this processor: try reducing\n");
    printf ("machine load.\n");
    
    exit (-1);
}
#endif

static inline us_time cycle_to_us (cycle_time c)
{
  return c / (us_time)MHZ;
}

static inline cycle_time us_to_cycle (us_time us)
{
  return (cycle_time)us * (cycle_time)MHZ;
}

void RTServer_busy::Periodic(void)
    throw (CORBA::SystemException)
{
    cycle_time last, curr, diff, total;

    total = us_to_cycle(this->rts_Compute.data[this->rts_Compute.index]);
    this->rts_Compute.index = ((this->rts_Compute.index + 1) %
			       this->rts_Compute.length);
    last = rdtsc();
    do {
	curr = rdtsc();
	diff = curr - last;
	if( diff > us_to_cycle(1.0) )
	{
	    // printf("gap\n");
	    curr = rdtsc();
	    /* gap... */
	}
	else
	{
	    total -= diff;
	}
	last = curr;
    } while( total > 0 );
}
