/*
 * GCTaskAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file GCTaskAdvocate.hh
 *
 * Header file for the GCTaskAdvocate class.
 */

#ifndef _gc_task_advocate_hh
#define _gc_task_advocate_hh

#include "ExactTaskAdvocate.hh"

class GCInterval
{
public:

    static const int TOLERANCE = 500;
    
    CORBA::ULong i_Compute;
    unsigned int i_Length;

    int match(CORBA::ULong ct)
    {
	int retval = 0;
	
	if( (ct > (this->i_Compute - TOLERANCE)) &&
	    (ct < (this->i_Compute + TOLERANCE)) )
	{
	    retval = 0;
	}
	return( retval );
    };
};
    
/**
 * An ExactTaskAdvocate that will always advise the gcimum required processing
 * time it has ever seen.
 */
class GCTaskAdvocate : public ExactTaskAdvocate
{

public:
    
    /** @copydoc Broker::TaskFactory::CreateTask */
    GCTaskAdvocate(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter);
    
    /**
     * Compute a new processing time advice from the given status value.  The
     * gc implementation will compare the advice reported by the parent class
     * against the current processing time and return the gcimum of the two.
     *
     * @param status The value passed into ReportCPU.
     * @return A processing time that will satisfy all requirements that have
     *         been encountered so far.
     * @exception InvalidStatus if the status value is invalid.
     * @exception CORBA::BAD_INV_ORDER if the method is called without
     *            BeginCPUScheduling() being called first.
     */
    virtual CORBA::ULong ComputeAdvice(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

protected:

    typedef enum {
	DETECT_MODE_OFF,
	DETECT_MODE_ON,
    } detect_mode_t;

    detect_mode_t gta_Mode;
    GCInterval gta_Intervals[16];
    unsigned int gta_Index;
    unsigned int gta_Length;
    unsigned int gta_PeriodIndex;

};

#endif
