/*
 * MaxDecayTaskAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file MaxDecayTaskAdvocate.hh
 *
 * Header file for the MaxDecayTaskAdvocate class.
 */

#ifndef _max_decay_task_advocate_hh
#define _max_decay_task_advocate_hh

#include "ExactTaskAdvocate.hh"

/**
 * An ExactTaskAdvocate that will always advise the maximum required processing
 * time it has ever seen.
 */
class MaxDecayTaskAdvocate : public ExactTaskAdvocate
{

public:
    
    /** @copydoc Broker::TaskFactory::CreateTask */
    MaxDecayTaskAdvocate(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter);
    
    /**
     * Compute a new processing time advice from the given status value.  The
     * max implementation will compare the advice reported by the parent class
     * against the current processing time and return the maximum of the two.
     *
     * @param status The value passed into ReportCPU.
     * @return A processing time that will satisfy all requirements that have
     *         been encountered so far.
     * @exception InvalidStatus if the status value is invalid.
     * @exception CORBA::BAD_INV_ORDER if the method is called without
     *            BeginCPUScheduling() being called first.
     */
    virtual CORBA::ULong ComputeAdvice(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

private:

    CORBA::ULong md_SubMax;
    unsigned int md_Decay;
    
};

#endif
