/*
 * GCTaskAdvocate.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file GCTaskAdvocate.cc
 *
 * Implementation of the GCTaskAdvocate class.
 */

#include "config.h"

#include "GCTaskFactory.hh"
#include "GCTaskAdvocate.hh"

template TaskFactoryTemplate<GCTaskAdvocate>;

static void timespec_to_microsec (struct timespec time_spec,
				  CORBA::ULong &microseconds)
{
    microseconds = time_spec.tv_sec * 1000000;
    microseconds += time_spec.tv_nsec / 1000;
}

GCTaskAdvocate::GCTaskAdvocate(const Broker::TaskParameters &tp) 
    throw (CORBA::SystemException,
	   Broker::DuplicateTaskParameter,
	   Broker::InvalidTaskParameter,
	   Broker::MissingTaskParameter) :
    ExactTaskAdvocate(tp), RKTask(tp)
{
    memset(this->gta_Intervals, 0, sizeof(this->gta_Intervals));
    this->gta_Index = 0;
    this->gta_Length = 0;
    this->gta_PeriodIndex = 0;
    this->gta_Mode = DETECT_MODE_ON;
}

CORBA::ULong GCTaskAdvocate::ComputeAdvice(CORBA::ULong status)
    throw (CORBA::SystemException, Broker::InvalidStatus)
{
    CORBA::ULong ct, retval;
    
    retval = this->ExactTaskAdvocate::ComputeAdvice(status);
    timespec_to_microsec(this->rkt_CPUReserveSpec.compute_time, ct);

    if( this->gta_PeriodIndex >= 8 )
    {
	this->gta_Intervals[this->gta_PeriodIndex] =
	    this->gta_Intervals[0];
	this->gta_Mode = DETECT_MODE_OFF;
	this->gta_PeriodIndex = 0;
    }
    
    switch( this->gta_Mode )
    {
    case DETECT_MODE_ON:
	this->gta_Intervals[this->gta_Index].i_Compute = retval;
	this->gta_Index += 1;
	this->gta_Length += 1;
	cout << " * learned: " << retval << endl;
	break;
    case DETECT_MODE_OFF:
	retval = this->gta_Intervals[this->gta_PeriodIndex + 1].i_Compute +
	    GCInterval::TOLERANCE;
	cout << " * suggest: " << retval << endl;
	break;
    }

    this->gta_PeriodIndex += 1;
    return( retval );
}
