/*
 * MaxDecayTaskAdvocate.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file MaxDecayTaskAdvocate.cc
 *
 * Implementation of the MaxDecayTaskAdvocate class.
 */

#include "config.h"

#include "MaxDecayTaskFactory.hh"
#include "MaxDecayTaskAdvocate.hh"

template TaskFactoryTemplate<MaxDecayTaskAdvocate>;

static void timespec_to_microsec (struct timespec time_spec,
				  CORBA::ULong &microseconds)
{
    microseconds = time_spec.tv_sec * 1000000;
    microseconds += time_spec.tv_nsec / 1000;
}

MaxDecayTaskAdvocate::MaxDecayTaskAdvocate(const Broker::TaskParameters &tp) 
    throw (CORBA::SystemException,
	   Broker::DuplicateTaskParameter,
	   Broker::InvalidTaskParameter,
	   Broker::MissingTaskParameter) :
    ExactTaskAdvocate(tp), RKTask(tp)
{
    this->md_Decay = 5;
}

CORBA::ULong MaxDecayTaskAdvocate::ComputeAdvice(CORBA::ULong status)
    throw (CORBA::SystemException, Broker::InvalidStatus)
{
    CORBA::ULong ct, retval;
    
    retval = this->ExactTaskAdvocate::ComputeAdvice(status);
    timespec_to_microsec(this->rkt_CPUReserveSpec.compute_time, ct);
    if( retval >= ct )
    {
	this->md_Decay = 5;
	this->md_SubMax = 0;
	retval = max(retval, ct);
    }
    else if( this->md_Decay > 0 )
    {
	this->md_SubMax = max(this->md_SubMax, retval);
	this->md_Decay -= 1;
	retval = max(retval, ct);
    }
    else if( this->md_Decay == 0 )
    {
	retval = (CORBA::ULong)(this->md_SubMax + (0.05 * this->md_SubMax));
	this->md_SubMax = 0;
	this->md_Decay = 5;
    }
    return( retval );
}
