/*
 * MaxTaskAdvocate.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file MaxTaskAdvocate.cc
 *
 * Implementation of the MaxTaskAdvocate class.
 */

#include "config.h"

#include "MaxTaskFactory.hh"
#include "MaxTaskAdvocate.hh"

template TaskFactoryTemplate<MaxTaskAdvocate>;

static void timespec_to_microsec (struct timespec time_spec,
				  CORBA::ULong &microseconds)
{
    microseconds = time_spec.tv_sec * 1000000;
    microseconds += time_spec.tv_nsec / 1000;
}

MaxTaskAdvocate::MaxTaskAdvocate(const Broker::TaskParameters &tp) 
    throw (CORBA::SystemException,
	   Broker::DuplicateTaskParameter,
	   Broker::InvalidTaskParameter,
	   Broker::MissingTaskParameter) :
    ExactTaskAdvocate(tp), RKTask(tp)
{
}

CORBA::ULong MaxTaskAdvocate::ComputeAdvice(CORBA::ULong status)
    throw (CORBA::SystemException, Broker::InvalidStatus)
{
    CORBA::ULong ct, retval;
    
    retval = this->ExactTaskAdvocate::ComputeAdvice(status);
    timespec_to_microsec(this->rkt_CPUReserveSpec.compute_time, ct);
    retval = max(retval, ct);
    return( retval );
}
