/*
 * exact_task_advocate.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file exact_task_advocate.cc
 *
 * Test the ExactTaskAdvocate class.
 */

#include "config.h"

/* Make sure assert is on */
#if !defined(DEBUG)
#define DEBUG 1
#endif

#include <assert.h>
#include <assert_pp.h>

#include <rk_stub.h>

#include <iostream>

#include <ManagerImpl.hh>

#include <ExactTaskFactory.hh>
#include <ExactTaskAdvocate.hh>

#include "StubPolicy.hh"

#if defined(HAVE_QUO)
#include <quo/QuoKernel.h>
#endif

int main(int argc, char *argv[])
{
    int retval = 0;
    
    rk_stub_set_mode(RK_STUB_LOG);

    /* ORB/POA setup */
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    
    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var mgr = root_poa->the_POAManager();
    mgr->activate();

    Broker::RealTimeAdvocate_var rta;
    Broker::ScheduleParameters sp;
    Broker::Manager_var manager;
    Broker::TaskFactory_var tf;
    Broker::TaskParameters tp;
    Broker::Policy_var policy;
    Broker::Task_ptr task;
    
    ManagerImpl mi("dummy");
    StubPolicy spi("dummy");

    manager = mi._this();
    policy = spi._this();

    manager->CurrentPolicy(policy.in());
    
    pid_t c1;

    c1 = rk_stub_mk_pid("test", NULL, NULL, NULL);
    
    ExactTaskFactory etf("ExactTaskAdvocate");

    tf = etf._this();

    tp.length(2);
    tp[0].name = "name";
    tp[0].value <<= "test";
    tp[1].name = "pid";
    tp[1].value <<= (CORBA::Long)c1;

    task = tf->CreateTask(tp);

    assert(strcmp(task->Name(), "test") == 0);

    rta = Broker::RealTimeAdvocate::_narrow(task);

    sp.length(1);
    sp[0].name = "period";
    sp[0].value <<= (CORBA::ULong)10000;
    
    rta->BeginCPUScheduling(manager.in(), sp);

    cout << "old ct = " << rta->GetComputeTime() << endl;
    try
    {
	rta->ReportCPU(10);
    }
    catch(const CORBA::SystemException &e)
    {
	cerr << "se = " << e << endl;
    }
    cout << "new ct = " << rta->GetComputeTime() << endl;

#if defined(HAVE_QUO)
    {
	QUO_ValueSCImpl missed_sci;
	quo::ValueSC_var missed_sc;
	
	missed_sc = missed_sci._this();

	cout << "sc = " << missed_sc->longValue() << endl;

	missed_sc->longValue(5);
    }
#endif
    
    rta->EndCPUScheduling();

    return( retval );
}
